; Getevent.sc

; This script contains procedures to demonstrate the GETEVENT command and
; show the contents of the dynamic array for various events.


PROC GetEventTest()
 PRIVATE startticks,idlflag

  idleflag = True


; - Section 1 - INITIALIZE THE DESKTOP -----------------------------------------

  ECHO NORMAL                                    ; Remove the full-screen canvas
                                                 ; to provide Desktop visibility
                                                 ; and allow mouse events to
                                                 ; work.
  MESSAGE "This is a test of the GETEVENT command"

  SetupGetEventDisplayEventsWindow()


  MESSAGE "Loading form, Please wait"            ; on the screen while form is
                                                 ; loading

; - Section 2 - PLACE TABLES ON DESKTOP, GET WINDOW HANDLE ---------------------

  VIEW "Employee"
  WINDOW HANDLE IMAGE IMAGENO() TO emptbl.h
  WINDOW MOVE emptbl.h TO 1,0
  WINDOW RESIZE emptbl.h TO 10,29

  VIEW "Dept"
  WINDOW HANDLE IMAGE IMAGENO() TO deptbl.h
  WINDOW MOVE deptbl.h TO 12,0
  WINDOW RESIZE deptbl.h TO 10,29


; - Section 3 - SHOWPULLDOWN AND PROMPT ----------------------------------------

  SHOWPULLDOWN
    "Employee"     : "Move to the Employee table"          : "TblEmp",
    "Dept"         : "Move to the Dept table"              : "TblDep",
    "Event ~T~ime" : "Change to delay between events"      : "SetDelay",
    "E~x~it"       : "Exit GETEVENT example"               : "ExitMnu"
      SUBMENU
        "No"       : "Do not exit, return to GETEVENT " +
                     "example"                             : "ExitNo",
        "Yes"      : "Save and exit GETEVENT example"      : "ExitYes"
      ENDSUBMENU
  ENDMENU
  msg   = ""
  PROMPT "[Ctrl-Q]-Exit   [F3]-Prev Table   [F4]-Next Table"
  MESSAGE msg


  ECHO NORMAL                                    ; Remove the full-screen canvas
                                                 ; to provide Desktop visibility
                                                 ; and allow mouse events to
                                                 ; work.

; - Section 4 - GETEVENT LOOP --------------------------------------------------

  WHILE True
    GETEVENT KEY "DOWN","UP",17,"F3","F4"
             MOUSE "UP","DOWN"
             MESSAGE "CLOSE","MAXIMIZE","NEXT","MENUSELECT"
             IDLE
    TO event_bag                                 ; Get the current event.
    IF event_bag["TYPE"] <> "IDLE" THEN
      GetEventDisplayEvents(sleepamt)
    ENDIF
    SWITCH
      CASE event_bag["TYPE"] = "KEY" :           ; If the event is a keypress
           IF event_bag["KEYCODE"] = 17 THEN     ; and it is Ctrl-Q then quit.
             retval = DlgYesNo("","","Are you sure you want to exit?","","AC")
             IF retval = "Yes" THEN
               QUITLOOP
             ELSE
               LOOP
             ENDIF
           ENDIF
      CASE event_bag["TYPE"] = "MESSAGE" :
           IF event_bag["MESSAGE"] = "CLOSE" THEN
             DlgMsg("WARNING!","","You cannot close the window!","","AC")
             LOOP
           ENDIF
           IF event_bag["MESSAGE"] = "MENUSELECT" THEN
             ms = event_bag["MENUTAG"]
             SWITCH
               CASE ms = "TblEmp"   : MOVETO "Employee"
               CASE ms = "TblDep"   : MOVETO "Dept"
               CASE ms = "SetDelay" : sleepamt = DlgSetEventDelay()
               CASE ms = "ExitYes"  :
                    retval = DlgYesNo("","",
                                      "Are you sure you want to exit?","","AC")
                    IF retval = "Yes" THEN
                      QUITLOOP
                    ELSE
                      LOOP
                    ENDIF
             ENDSWITCH
           ENDIF
    ENDSWITCH
    EXECEVENT event_bag                          ; Execute the event.
  ENDWHILE
  ECHO OFF
  ALTSPACE {Desktop} {Empty}                     ; Clear the Desktop

  QUIT "Let's now look at the code"

ENDPROC

PROC GetEventDisplayEvents(slp)
 PRIVATE element,                                ;  Variable for dynarray index
         kp                                      ;  Used with GETCHAR to pause

  SETCANVAS events.h                             ;  Set the current canvas to
                                                 ; the window to display the
                                                 ; the current event
  @ 0,0 CLEAR EOS
  ? "Contents of the dynamic array:"
  FOREACH element In event_bag
    ? "  [",element,"] = ",event_bag[element]
  ENDFOREACH
  ECHO NORMAL
  IF slp = 0 THEN                                ;  If you select to 'pause'
    MESSAGE "Press any key to continue..."       ; between events, display the
    kp = GETCHAR()                               ; message and wait for a key
    MESSAGE ""                                   ; else wait for the specified
  ELSE                                           ; number of seconds
    SLEEP slp
  ENDIF
ENDPROC


PROC DlgSetEventDelay()
 PRIVATE okcancel,
         delaytime
  delaytime = 2                                  ;  Initialize the radio button
                                                 ; to option number 2 which is
                                                 ; to wait 1 second
  SHOWDIALOG "Set value to pause between events"
     @ 5,5
     HEIGHT 10 WIDTH 42

     RADIOBUTTONS
       @ 1,1
       HEIGHT 5
       WIDTH 26
       "~W~ait for a key press",
       "~1~ second",
       "~2~ seconds",
       "~3~ seconds",
       "~4~ seconds"
       TAG "Delaytime"
     TO delaytime

     PUSHBUTTON @ 3,28
       WIDTH 10
       "~O~K"
       OK
       DEFAULT
       VALUE "OK"
       TAG "OK"
     TO okcancel

  ENDDIALOG
  RETURN (delaytime - 1) * 1000                  ;  Return the value that is
                                                 ; used for the SLEEP command.
ENDPROC


PROC SetupGetEventDisplayEventsWindow()
 PRIVATE wa_bag                                  ;  Dynarray for window
                                                 ; attributes

  ECHO NORMAL                                    ;  Show the desktop on the
  ECHO OFF                                       ; full-screen canvas

  DYNARRAY wa_bag[]                              ;  Define and initialize the
  wa_bag["HASSHADOW"]    = False                 ; dynarray used to set the
  wa_bag["MARGIN"]       = 1                     ; window attributes of the
  wa_bag["CANCLOSE"]     = False                 ; canvas windows where the
  wa_bag["CANMAXIMIZE"]  = False                 ; event information is
  wa_bag["CANRESIZE"]    = False                 ; displayed

  sleepamt = DlgSetEventDelay()                  ;  Get the time delay value
                                                 ; between each event

  wa_bag["TITLE"]     = "Event Information"      ;  Create the floating window
  wa_bag["CANVASHEIGHT"] = 22                    ; to display the events
  wa_bag["CANVASWIDTH"]  = 37
  WINDOW CREATE ATTRIBUTES wa_bag @ 1,40 HEIGHT 24 WIDTH 39 TO events.h

ENDPROC


PROC DlgYesNo(dlgtitle,line1,line2,line3,frmat)
 PRIVATE yesnotbutton,                           ;  Push button variable
         boxwidth,                               ;  Dialog box width
         l1len,                                  ;  Length of line 1
         l2len,                                  ;  Length of line 2
         l3len,                                  ;  Length of line 3
         sysinfo_bag                             ;  Dynamic array for SYSINFO
;        line1                                   ;  First line of text
;        line2                                   ;  Second line of text
;        line3                                   ;  Third line of text
;        frmat                                   ;  Alignment option of FORMAT

  SYSINFO TO sysinfo_bag                         ;  Get screen width & height
  l1len = LEN(line1)                             ;  Get the length of each
  l2len = LEN(line2)                             ; line of text
  l3len = LEN(line3)
  IF l1len > 76 THEN                             ;  If the length of any text
    line1 = SUBSTR(line1,1,76)                   ; is > 76 then truncate to
    l1len = 76                                   ; a maximum length of 76
  ENDIF                                          ; characters per line
  IF l2len > 76 THEN
    line2 = SUBSTR(line2,1,76)
    l2len = 76
  ENDIF
  IF l3len > 76 THEN
    line3 = SUBSTR(line3,1,76)
    l3len = 76
  ENDIF
  boxwidth = MAX(MAX(l1len,l2len),l3len) + 4     ;  Set the width of the dialog
                                                 ; box to the maximum line
                                                 ; length plus 2 cols for the
                                                 ; border and a single column
                                                 ; on each side of the line
  IF boxwidth < 22 THEN                          ;  The minimut box width is 22
    boxwidth = 22                                ; characters
  ENDIF

  IF SEARCH(UPPER(frmat),"AL!AC!AR") = 0 THEN    ;  Ensure the alignment format
    frmat = "AL"                                 ; is valid otherwise align the
  ENDIF                                          ; text on the left
  frmat = "W" + STRVAL(boxwidth - 4) +           ;  Set the width parameter for
          "," + frmat                            ; FORMAT

  yesnobutton = "No"                             ;  Initialize the yesnobutton
                                                 ; to No

  SHOWDIALOG dlgtitle
    @ INT((sysinfo_bag["ScreenHeight"]-8)/2),         ;  Center the dialog box
      INT((sysinfo_bag["ScreenWidth"] - boxwidth)/2)  ; in the center of the
    HEIGHT 8 WIDTH boxwidth                           ; screen
    @ 0,1 ?? FORMAT(frmat,line1)                 ;  Display the 3 lines of text
    @ 1,1 ?? FORMAT(frmat,line2)                 ; aligning them based on
    @ 2,1 ?? FORMAT(frmat,line3)                 ; the passed parameter

    PUSHBUTTON @ 4, INT((boxwidth-14-2)/3)       ;  The Yes and No buttons
      WIDTH 7                                    ; are centered in the dialog
      "~Y~es"                                    ; box based on the width of
      OK                                         ; the text passed.
      VALUE "Yes"
      TAG "yes"
    TO yesnobutton

    PUSHBUTTON @ 4, INT((boxwidth-14-2)/3) * 2 + 7
      WIDTH 7
      "~N~o"
      CANCEL
      VALUE "No"
      TAG "no"
    TO yesnobutton

  ENDDIALOG
  RETURN yesnobutton                             ;  If No or Esc pressed, return
                                                 ; "No" otherwise return "Yes"
ENDPROC

PROC DlgMsg(dlgtitle,line1,line2,line3,frmat)
 PRIVATE okbutton,                               ;  Push button variable
         boxwidth,                               ;  Dialog box width
         l1len,                                  ;  Length of line 1
         l2len,                                  ;  Length of line 2
         l3len,                                  ;  Length of line 3
         sysinfo_bag                             ;  Dynamic array for SYSINFO
;        line1                                   ;  First line of text
;        line2                                   ;  Second line of text
;        line3                                   ;  Third line of text
;        frmat                                   ;  Alignment option of FORMAT

  SYSINFO TO sysinfo_bag                         ;  Get screen width & height
  l1len = LEN(line1)                             ;  Get the length of each
  l2len = LEN(line2)                             ; line of text
  l3len = LEN(line3)
  IF l1len > 76 THEN                             ;  If the length of any text
    line1 = SUBSTR(line1,1,76)                   ; is > 76 then truncate to
    l1len = 76                                   ; a maximum length of 76
  ENDIF                                          ; characters per line
  IF l2len > 76 THEN
    line2 = SUBSTR(line2,1,76)
    l2len = 76
  ENDIF
  IF l3len > 76 THEN
    line3 = SUBSTR(line3,1,76)
    l3len = 76
  ENDIF
  boxwidth = MAX(MAX(l1len,l2len),l3len) + 4     ;  Set the width of the dialog
                                                 ; box to the maximum line
                                                 ; length plus 2 cols for the
                                                 ; border and a single column
                                                 ; on each side of the line
  IF boxwidth < 10 THEN                          ;  The minimum box width is 10
    boxwidth = 10                                ; characters
  ENDIF

  IF SEARCH(UPPER(frmat),"AL!AC!AR") = 0 THEN    ;  Ensure the alignment format
    frmat = "AL"                                 ; is valid otherwise align the
  ENDIF                                          ; text on the left
  frmat = "W" + STRVAL(boxwidth - 4) +           ;  Set the width parameter for
          "," + frmat                            ; FORMAT

  SHOWDIALOG dlgtitle
    @ INT((sysinfo_bag["ScreenHeight"]-8)/2),        ;  Center the dialog box
      INT((sysinfo_bag["ScreenWidth"] - boxwidth)/2) ; in the center of the
    HEIGHT 8 WIDTH boxwidth                          ; screen
    @ 0,1 ?? FORMAT(frmat,line1)                 ;  Display the 3 lines of text
    @ 1,1 ?? FORMAT(frmat,line2)                 ; aligning them based on
    @ 2,1 ?? FORMAT(frmat,line3)                 ; the passed parameter

    PUSHBUTTON @ 4, INT((boxwidth-2)/2) - 5          ;  Center the OK button
      WIDTH 6
      "~O~K"
      OK DEFAULT
      VALUE "OK"
      TAG "ok"
    TO okbutton

  ENDDIALOG
ENDPROC


GetEventTest()
